<!---
	DirectoryWatcher
	
	Watches any number of file system directories and notifies registered listeners
	when files are added, changed, or deleted from watched directories. Listeners
	must implement the DirectorListener interface.
	
	When creating a DirectoryWatcher instance, the init() method must be called to
	start the background thread; only one DirectoryWatcher may be created per
	BlueDragon instance.
	
	Sample usage:
	
		<cfset directoryWatcher = createObject( "component", "DirectoryWatcher" ).init()>
		<cfset myListener = createObject( "component", "MyDirectoryListener" )>
		<cfset directoryWatcher.addListener( "C:\temp", myListener )>
		
	The most natural place to create a DirectoryWatcher instance is within the
	onApplicationStart() event handler of your Application.cfc.
	
	Information and error messages are logged to the DirectoryWatcher.log file.
	
	This has been tested on BlueDragon 7.0 beta2.
	
	vgb	10/25/2006	Created
--->
<cfcomponent>
	
	<!---
		init( [interval] )
		
		Creates a background thread that runs at the specified interval. The "interval"
		is in seconds, and is optional; the default is 60 seconds. Only one background
		thread can be created (only one instance of DirectoryWatcher may be initialized).
	--->
	<cffunction name="init" access="public" returntype="void" output="false">
		<cfargument name="interval" type="numeric" required="false" default="60">
		
		<!--- don't allow more than one DirectoryWatcher thread --->
		<cfset var threads = getAllThreads()>
		<cfloop from="1" to="#arrayLen( threads )#" index="i">
			<cfif threads[ i ].name eq "DirectoryWatcher">
				<cfthrow message="DirectoryWatcher thread already running">
			</cfif>
		</cfloop>
		
		<cfset this.listeners = StructNew()>
		<cfset this.directories = StructNew()>
		
		<!--- create a new background thread that runs forever --->
		<cfthread name="DirectoryWatcher" interval="#arguments.interval#">
			<cftry>
				<cfloop condition="true">
					<cfset this.checkDirectories()>
					<cfpause interval="#attributes.interval#">
				</cfloop>
				
				<cfcatch>
					<cflog file="DirectoryWatcher" type="error"
						   text="background thread terminated due to error: #cfcatch.message#">
					<!--- rethrow for "upstream" error handling --->
					<cfrethrow>
				</cfcatch>
			</cftry>
		</cfthread>
		
		<cflog file="DirectoryWatcher" text="background thread started">
		
		<cfreturn this>
	</cffunction>
	
	
	<!---
		addListener( directoryPath, listener )
		
		The "directoryPath" parameter must be the full physical path to a file system
		directory, must end with a file separator (either "/" or "\"), and the specified
		directory must actually exist.
		
		The "listener" parameter must be a CFC that implements the DirectoryListener
		interface.
	--->
	<cffunction name="addListener" access="public" returntype="void" output="false">
		<cfargument name="directoryPath" type="string" required="true">
		<cfargument name="listener" type="DirectoryListener" required="true">
		
		<cfif not isDefined( "this.listeners" )>
			<cfthrow message="instance not initialized">
		</cfif>
		
		<cfif not directoryExists( directoryPath )>
			<cfthrow message="directory does not exist">
		</cfif>
		
		<cfif not arguments.directoryPath.endsWith( "/" ) and
			  not arguments.directoryPath.endsWith( "\" )>
			  <cfthrow "directoryPath must end with file separator">
		</cfif>
		
		<cfset this.listeners[ directoryPath ] = arguments.listener>
		
		<!--- get the current list of files in the directory --->
		<cfset this.directories[ directoryPath ] = this.getFileList( directoryPath )>
		
		<cflog file="DirectoryWatcher" text="Added listener #getMetaData( listener ).name# for directory #directoryPath#">
	</cffunction>
	
	
	<!---
		removeListener( directoryPath )
		
		The "directoryPath" parameter must be the same physical directory path specified
		in a previous call to addListener( directoryPath, listener ), however, no checks
		are made to insure this.
	--->
	<cffunction name="removeListener" access="public" returntype="void" output="false">
		<cfargument name="directoryPath" type="string" required="true">
		
		<cfif not isDefined( "this.listeners" )>
			<cfthrow message="instance not initialized">
		</cfif>
		
		<cfset structDelete( this.directories, arguments.directoryPath )>
		<cfset structDelete( this.listeners, arguments.directoryPath )>
		
		<cflog file="DirectoryWatcher" text="Removed listener for directory #directoryPath#">
	</cffunction>
	
	
	<!------------------------->
	<!--- PRIVATE FUNCTIONS --->
	<!------------------------->
	
	
	<!---
		checkDirectories()
		
		Loop over all directories in the "this.directories" struct, checking each one
		for added, changed, or deleted files, notifying the listener in each case.
	--->
	<cffunction name="checkDirectories" access="private" returntype="void" output="false">
		<cfloop list="#structKeyList( this.directories )#" index="directoryPath">
			<cfset this.checkDirectory( directoryPath )>
		</cfloop>
	</cffunction>
	
	
	<!---
		checkDirectory( directoryPath )
		
		Checks for added, changed, or deleted files in the specified directory, notifying
		the listener in each case. 
	--->
	<cffunction name="checkDirectory" access="private" returntype="void" output="false">
		<cfargument name="directoryPath" type="string" required="true">
		
		<cfassert structKeyExists( this.listeners, arguments.directoryPath )>
		<cfassert structKeyExists( this.directories, arguments.directoryPath )>
		
		<cfset var listener = this.listeners[ arguments.directoryPath ]>
		<cfset var previousFiles = this.directories[ arguments.directoryPath ]>
		<cfset var currentFiles = this.getFileList( arguments.directoryPath )>
		
		<!--- loop over the current files and check for changed or new files --->
		<cfloop list="#structKeyList( currentFiles )#" index="currentFile">
			<cfif structKeyExists( previousFiles, currentFile )>
				<cfif previousFiles[ currentFile ] neq currentFiles[ currentFile ]>
					<!--- notify listener of file change --->
					<cfset this.onChange( listener, arguments.directoryPath & currentFile,
											currentFiles[ currentFile ] )>
				</cfif>
				<cfset structDelete( previousFiles, currentFile )>
			<cfelse>
				<!--- notify listener of new file --->
				<cfset this.onAdd( listener, arguments.directoryPath & currentFile,
									currentFiles[ currentFile ] )>
			</cfif>
		</cfloop>
		
		<!--- any files still in previousFiles struct have been deleted --->
		<cfloop list="#structKeyList( previousFiles )#" index="previousFile">
			<!--- notify listener of file deleted --->
			<cfset this.onDelete( listener, arguments.directoryPath & previousFile )>
		</cfloop>
		
		<!--- save the current file list for next check --->
		<cfset this.directories[ arguments.directoryPath ] = currentFiles>
	</cffunction>
	
	
	<!---
		onAdd( listener, fileName, lastModified )
		
		Notify the listener of file added. Catch and log all exceptions.
	--->
	<cffunction name="onAdd" access="private" returntype="void" output="false">
		<cfargument name="listener" type="DirectoryListener" required="true">
		<cfargument name="fileName" type="string" required="true">
		<cfargument name="lastModified" type="date" required="true">
		
		<cftry>
			<!--- notify listener of new file --->
			<cfset arguments.listener.onAdd( arguments.fileName, arguments.lastModified )>
			
			<cfcatch>
				<cflog file="DirectoryWatcher" type="error"
					   text="onAdd() exception: #cfcatch.message# #cfcatch.detail#, listener: #getMetaData( arguments.listener ).name#">
			</cfcatch>
		</cftry>
	</cffunction>
	

	<!---
		onChange( listener, fileName, lastModified )
		
		Notify the listener of file changed. Catch and log all exceptions.
	--->
	<cffunction name="onChange" access="private" returntype="void" output="false">
		<cfargument name="listener" type="DirectoryListener" required="true">
		<cfargument name="fileName" type="string" required="true">
		<cfargument name="lastModified" type="date" required="true">
		
		<cftry>
			<!--- notify listener of new file --->
			<cfset arguments.listener.onChange( arguments.fileName, arguments.lastModified )>
			
			<cfcatch>
				<cflog file="DirectoryWatcher" type="error"
					   text="onChange() exception: #cfcatch.message# #cfcatch.detail#, listener: #getMetaData( arguments.listener ).name#">
			</cfcatch>
		</cftry>
	</cffunction>
	

	<!---
		onDelete( listener, fileName, lastModified )
		
		Notify the listener of file deleted. Catch and log all exceptions.
	--->
	<cffunction name="onDelete" access="private" returntype="void" output="false">
		<cfargument name="listener" type="DirectoryListener" required="true">
		<cfargument name="fileName" type="string" required="true">
		
		<cftry>
			<!--- notify listener of new file --->
			<cfset arguments.listener.onDelete( arguments.fileName )>
			
			<cfcatch>
				<cflog file="DirectoryWatcher" type="error"
					   text="onDelete() exception: #cfcatch.message# #cfcatch.detail#, listener: #getMetaData( arguments.listener ).name#">
			</cfcatch>
		</cftry>
	</cffunction>
	
	
	<!---
		getFileList( directoryPath )
		
		Given the full physical path to a directory, return a struct containing all the
		files in the directory. Struct key is the file name, struct value is the file
		modification date.
	--->
	<cffunction name="getFileList" access="private" returntype="struct" output="false">
		<cfargument name="directoryPath" type="string" required="true">
		
		<cfassert directoryExists( directoryPath )>
		
		<cfset var dirList = "">
		<cfdirectory action="list" directory="#arguments.directoryPath#" recurse="false"
						filter="*" sort="asc" name="dirList">
						
		<!--- copy the cfdirectory results to a struct with filename as key --->
		<cfset var fileList = StructNew()>
		
		<cfloop query="dirList">
			<cfif dirList.type eq "file">
				<cfset fileList[ dirList.name ] = dirList.dateLastModified>
			</cfif>
		</cfloop>
		
		<cfreturn fileList>
	</cffunction>

</cfcomponent>