<!---
	ImageListener
	
	This CFC is a modified version of the imagewatcher.cfc created by Ray Camden as an
	example of using the CFMX 7 DirectoryWatcher gateway:
	
	   http://ray.camdenfamily.com/index.cfm/2006/10/13/Using-ColdFusions-DirectoryWatcher-Gateway
	
	Ray's original CFC has been modified to work with the BlueDragon 7.0 DirectoryWatcher.cfc
	as follows:
	
	   1. This CFC now implements the DirectoryListener.cfc interface; the IMPLEMENTS
	      attribute has been added to the CFCOMPONENT tag, and the onAdd() method
	      signature has been modified accordingly.
	      
	   2. The onChange() function also checks for correct file size (Ray's example doesn't
	      implement the onChange method).
	      
	   3. Image resizing is done using BlueDragon's built-in CFIMAGE tag instead of the
	      open source ImageCFC used by Ray. The resize logic has been modified to only
	      resize once if both height and width are too big.
	      
	   4. The onDelete() method just logs the fact that a file has been deleted (Ray's
	      example doesn't implement onDelete).
	      
	This has been tested on BlueDragon 7.0 beta2.
	
	vgb	10/25/2006	Created
--->
<cfcomponent implements="DirectoryListener">
	
	<!---
		onAdd( fileName, lastModified )
		
		Invoked when a file is added to a watched directory. The "fileName" argument
		contains the full physical path to the added file; the "lastModifiedDate"
		argument contains the file modification date.
		
		Make sure the file is an image file; delete it if it's not. Then check the
		image file size, resizing it (smaller) if it's too big.
	--->
	<cffunction name="onAdd" access="public" returntype="void" output="false">
		<cfargument name="fileName" type="string" required="true">
		<cfargument name="lastModified" type="date" required="true">
		
		<cflog file="ImageListener" text="file added: #arguments.fileName#">
		
		<cfif not listFindNoCase( "jpg,gif,png", listLast( filename, "." ) )>
			<cflog file="ImageListener" text="Deleting #arguments.filename# since it wasn't an image.">
			<cffile action="delete" file="#arguments.filename#">
			<cfreturn>
		</cfif>
		
		<cfset this.checkImageFileSize( arguments.fileName )>
	</cffunction>
	
	
	<!---
		onChange( fileName, lastModified )
		
		Invoked when a file in a watched directory is modified. The "fileName" argument
		contains the full physical path to the modified file; the "lastModifiedDate"
		argument contains the file modification date.
		
		Check the image file size, resizing it (smaller) if it's too big.
	--->
	<cffunction name="onChange" access="public" returntype="void" output="false">
		<cfargument name="fileName" type="string" required="true">
		<cfargument name="lastModified" type="date" required="true">
		
		<cflog file="ImageListener" text="file changed: #arguments.fileName#">
		<cfset this.checkImageFileSize( arguments.fileName )>
	</cffunction>
	
	
	<!---
		onDelete( fileName )
		
		Invoked when a file is deleted from a watched directory. The "fileName" argument
		contains the full physical path to the deleted file.
		
		Just log the fact that the file was deleted.
	--->
	<cffunction name="onDelete" access="public" returntype="void" output="false">
		<cfargument name="fileName" type="string" required="true">
		<cflog file="ImageListener" text="file deleted: #arguments.fileName#">
	</cffunction>
	

	<!------------------------->
	<!--- PRIVATE FUNCTIONS --->
	<!------------------------->
	
	
	<!---
		checkImageFileSize( fileName )
		
		First make sure the file contains a valid image; delete it if it doesn't. Then check the
		file size and resize it (smaller) if it's too big.
	--->
	<cffunction name="checkImageFileSize" access="private" returntype="void" output="false">
		<cfargument name="fileName" type="string" required="true">
		
		<cfimage action="info" srcfile="#arguments.fileName#">
		<cfif not cfimage.success>
			<cflog file="ImageListener" text="Deleting #arguments.fileName# due to error: #cfimage.errortext#">
			<cffile action="delete" file="#arguments.fileName#">
			<cfreturn>
		</cfif>
		
		<cflog file="ImageListener" text="#arguments.fileName# is a valid image">
		
		<cfset var resize = false>
		
		<!--- too wide? --->
		<cfif cfimage.width gt 250>
			<cfset resize = true>
			<cflog file="ImageListener" text="Resizing #arguments.fileName# because width was #cfimage.width#">
			<cfset cfimage.width = 250>
		</cfif>
		
		<!--- too high? --->
		<cfif cfimage.height gt 250>
			<cfset resize = true>
			<cflog file="ImageListener" text="Resizing #arguments.fileName# because height was #cfimage.height#">
			<cfset cfimage.height = 250>
		</cfif>
		
		<cfif resize>
			<!--- resize the image, overwriting the original file --->
			<cfset destFileName = GetDirectoryFromPath(arguments.fileName) & ListFirst(GetFileFromPath(arguments.fileName), ".") & "_resized." & ListLast(GetFileFromPath(arguments.fileName), ".")>
			<cfimage action="edit" srcfile="#arguments.fileName#" destfile="#destFileName#" nameconflict="overwrite" width="#cfimage.width#" height="#cfimage.height#">
		</cfif>
	</cffunction>

</cfcomponent>